/**
 * Copyright (c) 2022 Luca Magrone
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "save.h"
#include "download.h"
#include "../unbound-adblocker.h"

extern char adlist_hosts_url[MAX_URL];
extern char hosts_filename[MAX_FILENAME];
extern int hosts_is_local;
extern char redirect_string[MAX_REDIRECT];
extern int redirect;

static char placeholder[] = "00000000";
void ua_print_banner(FILE *fp)
{
    time_t rawtime;
    struct tm *current;
    time(&rawtime);
    current = localtime(&rawtime);


    if(fp != NULL) {
        fprintf(fp, "# Generated by unbound-adblocker on %s#\n# Total domains: %s\n#\n", asctime(current), placeholder);
        if (hosts_is_local)
        {
            fprintf(fp, "# Domains have been read from a file named \'%s\'\n\n", hosts_filename);
        }
        else
        {
            fprintf(fp, "# Domains have been downloaded from %s\n\n", adlist_hosts_url);
        }
    }
    else {
        fprintf(stderr, "error: the file must be opened before calling ua_print_banner()\n");
        exit(EXIT_FAILURE);
    }
}

void ua_fill_banner(FILE *fp, unsigned int domains)
{
    int len = strlen(placeholder) + 1;
    char *tmp_str = malloc(len);
    char tmp;

    if(fp != NULL) {
        rewind(fp);

        while(1) {
            tmp = fgetc(fp);
            if(feof(fp))
                break;

            if(tmp == EOF) {
                fprintf(stderr, "error: the file must be opened for read and write before calling ua_fill_banner()\n");
                exit(EXIT_FAILURE);
            }

            if(tmp == '0') {
                fseek(fp, -1, SEEK_CUR);
                fgets(tmp_str, len, fp);

                if (strncmp(tmp_str, placeholder, len - 1) == 0)
                {
                    fseek(fp, - (len - 1), SEEK_CUR);

                    fprintf(fp, "%u", domains);

                    while(1) {
                        tmp = fgetc(fp);
                        if(tmp == '\n')
                            break;

                        fseek(fp, -1, SEEK_CUR);
                        fputc(' ', fp);
                    }

                    break;
                }
            }
        }

        if(feof(fp)) {
            fprintf(stderr, "An unexpected error occured\n");
            free(tmp_str);
            exit(EXIT_FAILURE);
        }

    }
    else {
        fprintf(stderr, "error: the file must be opened before calling ua_fill_banner()\n");
        free(tmp_str);
        exit(EXIT_FAILURE);
    }

    free(tmp_str);
}

unsigned int ua_hosts_to_unbound_file(struct raw_download raw_download, char filename[])
{
    FILE *f;
    size_t count = 0, tmp_count = 0;
    char *tmp;
    char *tmp_realloc;
    size_t tmp_alloc = 0;
    unsigned int domains = 0;

    size_t i, j;

    f = fopen(filename, "w+");
    if(f == NULL) {
        fprintf(stderr, "error: could not open %s\n", filename);
        return 0;
    }

    ua_print_banner(f);

    tmp = NULL;

    while(count < raw_download.size) {
        if(tmp_count >= tmp_alloc) {
            tmp = realloc(tmp, tmp_alloc + 20);
            if(tmp == NULL) {
                fprintf(stderr, "Could not reallocate memory\n");
                return domains;
            }

            tmp_alloc += 20;
        }

        tmp[tmp_count] = raw_download.mem[count];
        tmp_count += 1;

        if(tmp[tmp_count - 1] == '\n' || tmp[tmp_count - 1] == 0) {
            tmp[tmp_count - 1] = 0;
            tmp_count = 0;

            // perform checks on tmp
            if(strlen(tmp) >= 7 && strncmp(tmp, "0.0.0.0", 7) == 0) {
                i = 8;
                // make sure i indicats a letter
                while(tmp[i] == ' ')
                    i += 1;

                // set j to the first whitespace or to the end
                j = i;
                while(1) {
                    if(tmp[j] != ' ' && tmp[j] != 0)
                        j += 1;
                    else
                        break;
                }

                // make sure the string ends at j
                tmp[j] = 0;

                if(strlen(&tmp[i]) >= 7 && strncmp(&tmp[i], "0.0.0.0", 7) != 0) {
                    if(redirect)
                        fprintf(f, "local-zone: \"%s\" redirect\n"
                                   "local-data: \"%s %s\"\n",
                                   &tmp[i], &tmp[i], redirect_string);
                    else
                        fprintf(f, "local-zone: \"%s\" always_nxdomain\n", &tmp[i]);
                    domains += 1;
                }
            }
        }

        count += 1;
    }

    free(tmp);

    ua_fill_banner(f, domains);

    fclose(f);

    return domains;
}
